/*

Plot.java 
  -- classes implementing the 2d constraint plotting window of IAsolver,
     as part of the "ia_solver library" version 0.1beta1, 10/97

Copyright (C) 1997 Timothy J. Hickey

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA  02111-1307, USA.

 */


package ia_solver;


import java.awt.*;
import java.util.*;

import ia_math.*;
import ia_parser.*;


/**
 * this class manages a window for plotting 2d constraint projections.
 */
public class Plot extends Frame
{
 int max_narrow = 1;
 String xString;
 String yString;
 int depth;
 Exp exp;
 RealIntervalTable vars;
 RealIntervalTable newVars = new RealIntervalTable();
 Interface gui;
 PlotCanvas c;
 boolean newPlot=true;
 Thread t;
 boolean zoomFlag;

 Choice depthChoice;
 Choice fatChoice;

 TextField numnarrows;
 TextField messages;
 Label xposition;
 Label yposition;


 Button buttonQuit;
 Button buttonRedraw;
 Button buttonNarrow;
 Button buttonZoom;
 Button buttonUnZoom;

 TextField xrangelabel;
 TextField yrangelabel;

 GridBagLayout gridbag = new GridBagLayout();

 RealInterval xint,yint;
 String xrange,yrange;

public Plot(String xx, String yy, 
            int dd, Exp eexp, RealIntervalTable vvars, 
            Interface the_gui)
	    //            SolverGUI the_gui)
  {


    xString=xx;
    yString=yy;
    depth=dd;
    exp=eexp;
    vars=vvars;
    gui = the_gui;
    setTitle("Plot: ("+xx+","+yy+")");


    if (exp==null) {
      gui.setMessageField("Syntax Error.....");
      this.pack();
      this.dispose();
      return;
    }

    xint= (vars.lookup(xx));
    yint= (vars.lookup(yy));
    if (xint==null) {
      gui.setMessageField("ERROR in Plot: unknown variable "+xx);
      this.pack();
      this.dispose(); return;
    }
    if (yint==null){
      gui.setMessageField("ERROR in Plot: unknown variable "+yy);
      this.pack();
      this.dispose();
      return;
    }


    gui.buttonEval.disable();
    gui.buttonAbsolve.disable();
    gui.buttonClearVar.disable();
    gui.buttonClearCon.disable();
    gui.constraints.setEditable(false);

    gui.plotViews.addElement(this);

    xrange= "["+xint.lo()+","+xint.hi()+"]";
    yrange= "["+yint.lo()+","+yint.hi()+"]";

    System.out.println("gui  = "+gui);
    System.out.println("exp  = "+exp);
    System.out.println("vars = \n"+vars);
    System.out.println("x = "+xString);
    System.out.println("y = "+yString);
    System.out.println("d = "+depth);


    c = new PlotCanvas(this);


    buttonQuit = new Button("Close");
    buttonRedraw = new Button("Refresh");
    buttonNarrow = new Button("Size to Fit");
    buttonZoom = new Button("Zoom");
    buttonUnZoom = new Button("UnZoom");
    buttonUnZoom.disable();
    messages = new TextField("",20);
    messages.setEditable(false);

    xrangelabel = new TextField(xrange+"               ");
    yrangelabel = new TextField(yrange+"               ");
    xrangelabel.setEditable(false);
    yrangelabel.setEditable(false);

    xposition = new Label(xrange);
    yposition = new Label(yrange);



    fatChoice = new Choice();
    fatChoice.addItem("Shade boundary");
    fatChoice.addItem("Don't shade boundary");
    fatChoice.select(0);

    depthChoice = new Choice();
    depthChoice.addItem("0");
    depthChoice.addItem("1");
    depthChoice.addItem("2");
    depthChoice.addItem("3");
    depthChoice.addItem("4");
    depthChoice.addItem("5");
    depthChoice.addItem("6");
    depthChoice.addItem("7");
    depthChoice.addItem("8");
    depthChoice.addItem("9");
    depthChoice.addItem("10");
    depthChoice.select(depth);

    numnarrows = new TextField(""+max_narrow,6);

    Panel buttonPanel = new Panel();
    buttonPanel.setLayout(new GridLayout(4,1,5,5));
    buttonPanel.add(buttonQuit);
    buttonPanel.add(buttonNarrow);
    //    buttonPanel.add(buttonRedraw);
    buttonPanel.add(buttonZoom);
    buttonPanel.add(buttonUnZoom);
   

    Panel p = new Panel();
    p.setLayout(gridbag);
    Constrain.constrain(p,buttonPanel,0,0,1,4,GridBagConstraints.NONE,GridBagConstraints.WEST,0.0,0.0,5,5,5,25);
    //    Constrain.constrain(p,buttonPanel,0,0,2,1);

    // row 0, col 1-2 
    Constrain.constrain(p,new Label("  depth"),1,0,1,1,GridBagConstraints.NONE,GridBagConstraints.EAST,0.0,0.0, 0,0,0,0);
    Constrain.constrain(p,depthChoice,2,0,1,1);


    // row 0, col 3-4
    Constrain.constrain(p,new Label(xString+" axis"),
    3,0,1,1,      GridBagConstraints.NONE,GridBagConstraints.EAST,0.0,0.0, 0,0,0,0);
    Constrain.constrain(p,xrangelabel,
    4,0,1,1,GridBagConstraints.HORIZONTAL,GridBagConstraints.WEST,1.0,1.0, 0,0,0,0);

    // row 1, col 3-4
    Constrain.constrain(p,new Label(yString+" axis"),
    3,1,1,1,      GridBagConstraints.NONE,GridBagConstraints.EAST,0.0,0.0, 0,0,0,0);
    Constrain.constrain(p,yrangelabel,
    4,1,1,1,GridBagConstraints.HORIZONTAL,GridBagConstraints.WEST,1.0,1.0, 0,0,0,0);

    // row 2, col 3-4
    Constrain.constrain(p,new Label("cursor "+xString+" val"),
    3,2,1,1,      GridBagConstraints.NONE,GridBagConstraints.EAST,0.0,0.0, 0,0,0,0);
    Constrain.constrain(p,xposition,
    4,2,1,1,GridBagConstraints.HORIZONTAL,GridBagConstraints.WEST,0.0,0.0, 0,0,0,0);

    // row 3, col 3-4
    Constrain.constrain(p,new Label("cursor "+yString+" val"),
    3,3,1,1,      GridBagConstraints.NONE,GridBagConstraints.EAST,0.0,0.0, 0,0,0,0);
    Constrain.constrain(p,yposition,
    4,3,1,1,GridBagConstraints.HORIZONTAL,GridBagConstraints.WEST,0.0,0.0, 0,0,0,0);

    // row 4, col 1-4
    Constrain.constrain(p,new Label("Messages:"),
    1,4,1,1,      GridBagConstraints.NONE,GridBagConstraints.EAST,0.0,0.0, 0,0,0,0);
    Constrain.constrain(p,              messages,
    2,4,3,1,GridBagConstraints.HORIZONTAL,GridBagConstraints.WEST,0.0,0.0,5,5,5,5);

    // row 5, col 1-2
    Constrain.constrain(p,new Label("  max_narrows"),
    1,5,1,1,GridBagConstraints.NONE,GridBagConstraints.EAST,0.0,0.0, 0,0,0,0);
    Constrain.constrain(p,                numnarrows,
    2,5,1,1,GridBagConstraints.NONE,GridBagConstraints.WEST,0.0,0.0, 0,0,0,0);

    // row 5, col 3-4 
    Constrain.constrain(p,new Label("  shading"),
    3,5,1,1,GridBagConstraints.NONE,GridBagConstraints.EAST,0.0,0.0, 0,0,0,0);
    Constrain.constrain(p,             fatChoice,
    4,5,1,1,GridBagConstraints.NONE,GridBagConstraints.WEST,0.0,0.0, 0,0,0,0);

    this.setLayout(gridbag);
    Constrain.constrain(this,c,
    0,0,1,1,GridBagConstraints.BOTH,GridBagConstraints.EAST,2.0,2.0,5,5,5,5);
    Constrain.constrain(this,p,
    0,1,1,1,GridBagConstraints.NONE,GridBagConstraints.WEST,0.0,0.0,5,5,5,5);

    this.pack();
    this.show();

  }

public boolean handleEvent(Event evt)
  {
    //    System.out.println("PLOT event: "+evt);
    if(evt.id == Event.WINDOW_DESTROY) this.dispose();
    return super.handleEvent(evt);
  }
  
void resetPlot() {
      messages.setText("resetting Plot");
      vars = gui.getVars();
      exp = gui.getConstraints();

      xint= (vars.lookup(xString));
      yint= (vars.lookup(yString));
      xrange= "["+xint.lo()+","+xint.hi()+"]";
      yrange= "["+yint.lo()+","+yint.hi()+"]";

      xrangelabel.setText(xrange);
      yrangelabel.setText(yrange);
      gui.notifyNarrows(0);
      depth = 3;
      newPlot=true;
      c.restart();
      t = new Thread(c);
      //      buttonRedraw.disable();
      buttonNarrow.disable();
      buttonZoom.disable();
      if (gui.sizeStack() == 0) {
         buttonUnZoom.disable();
      } else {
         buttonUnZoom.enable();
      }
      t.start();
      messages.setText("Zoom depth: "+gui.sizeStack());
  }


void resetAllPlots() {
  for (int i = 0; i<gui.plotViews.size(); i++)
    ((Plot) gui.plotViews.elementAt(i)).resetPlot();
}

public boolean action(Event evt, Object arg)
  {
    //    System.out.println("PLOT actionevent: "+evt+" "+arg);


    if (evt.target == depthChoice) {
      int tmp;
      tmp = Integer.parseInt(depthChoice.getSelectedItem());
      //      if (tmp <= depth) {
      //        messages.setText("Depth "+tmp+" has already been computed");
      //        return true;
      //      }
      depth = tmp;
      max_narrow = Integer.parseInt(numnarrows.getText());
      newPlot=true;
      t = new Thread(c);
      //      buttonRedraw.disable();
      buttonNarrow.disable();
      buttonZoom.disable();
      buttonUnZoom.disable();
      t.start();
    }
    if (evt.target == fatChoice) {
      newPlot=false;
      t = new Thread(c);
      t.start();
    }
    else if (arg.equals("Close")) {
      if (t != null) t.stop();
      gui.plotViews.removeElement(this);
      if (gui.plotViews.size()==0) {
        gui.varStack = new Vector();
        gui.constraintStack=new Vector();
        gui.buttonEval.enable();
        gui.buttonAbsolve.enable();
	gui.buttonClearVar.enable();
	gui.buttonClearCon.enable();
        gui.constraints.setEditable(true);
      }
      this.dispose();
    }
    else if (arg.equals("Zoom")) {

      messages.setText("Use the mouse to select an area to zoom into");
      //      buttonZoom.setLabel("Unzoom");
      zoomFlag = true;
    }
    else if (arg.equals("UnZoom")) {
      gui.popStack();
      messages.setText("unzooming...");
      resetAllPlots();
    }
    else if (arg.equals("Refresh")) {
      Thread t = new Thread(c);
      t.start();
    }
    else if (arg.equals("Size to Fit")) {
      gui.vars.intersect(newVars);
      resetAllPlots();
      messages.setText("Graph resized and narrowed to depth 3");
      /*
      xrange= (vars.lookup(xString)).toString();
      yrange= (vars.lookup(yString)).toString();
      xrangelabel.setText(xrange);
      yrangelabel.setText(yrange);
      gui.notifyNarrows(0);
      depth = 3;
      newPlot=true;
      c.restart();
      t = new Thread(c);
//      buttonRedraw.disable();
      buttonNarrow.disable();
      buttonZoom.disable();
      buttonUnZoom.disable();
      t.start();
      */
    }
    return super.action(evt,arg);
  }
  
  //public Dimension preferredSize() {
  //    return new Dimension(400,400);
  //}


public void update(Graphics g) {
  paint(g);
}

public void paint(Graphics g) {
  super.paint(g);
  Thread t = new Thread(c);
  t.start();
}

public boolean solvable(int i,int j,int depth) {
  RealIntervalTable myVars = (RealIntervalTable) vars.clone();

  RealInterval xval,yval;
  double xlo,xhi,ylo,yhi,height,width;
  double newxlo,newxhi,newylo,newyhi;

  xval = myVars.lookup(xString);
  yval = myVars.lookup(yString);
  xlo = xval.lo();
  xhi = xval.hi();
  height = xhi-xlo;

  newxlo = xlo + i*(height/depth);
  newxhi = xlo + (i+1)*(height/depth);

  xval = new RealInterval(newxlo,newxhi);
  myVars.store(xString,xval);


  ylo = yval.lo();
  yhi = yval.hi();
  width=yhi-ylo;
  if (Double.isInfinite(width)) return true;

  newylo = ylo + j*(width/depth);
  newyhi = ylo + (j+1)*(width/depth);

  yval = new RealInterval(newylo,newyhi);
  myVars.store(yString,yval);

  //    System.out.println("\nplot solve for ("+i+","+j+")");
  //    System.out.println(" myVars= "+myVars);
  //    System.out.println("    exp= "+exp);

  exp.bindVars(myVars);

  Interpreter interpreter = new Interpreter(gui);
  boolean result =  interpreter.narrow(exp,max_narrow,false);

  //  System.out.println("("+result+","+xval+","+yval+")");

  if (result) {
    newVars.union(myVars);
    //    System.out.println("\n\nIn    solvable("+i+","+j+"):\n myVars =\n"+myVars+"\n newVars= "+newVars);
  }
  else {
    //    System.out.println("\n\n  NOT solvable("+i+","+j+"):\n vars=\n"+vars+"\nmyVars =\n"+myVars+"\n newVars= "+newVars);
  }

  return  result;
}




  };               



class Solution {
  int i;
  int j;
  Solution(int a, int b) {
  i=a; j=b; 
  }
}

class PlotCanvas extends Canvas implements Runnable {

 Plot parent;
 Vector solution_set = new Vector();
 Enumeration enum = null;



 Vector[] sol_set = new Vector[20];
 int[] depthsize = {1,2,4,8,16,32,64,128,256,512,1024,2048,4096};
 int plotsize=1;
 int current_depth=0;

 Dimension canvas_size;
 Dimension plot_size = new Dimension();
 int boxsize_x;
 int boxsize_y; 
 int two_to_depth=1;

 int zoom_x,zoom_y,last_x,last_y;

PlotCanvas(Plot f) {
    parent = f;
    solution_set = new Vector();
    solution_set.addElement(new Solution(0,0));   
    for (int i = 0; i<20; i++) 
      sol_set[i]=null;
    sol_set[0] = solution_set;
}


public void restart()
  {
   solution_set = new Vector();
   solution_set.addElement(new Solution(0,0));   
    for (int i = 0; i<20; i++) 
      sol_set[i]=null;
    sol_set[0] = solution_set;
   current_depth = 0;
   plotsize = 1;
   parent.depthChoice.select(parent.depth);
  }

public Dimension preferredSize() {
    return new Dimension(200,200);
}


void show_position(int i, int j) {
    double xratio,yratio;
    double width,height;
    RealInterval   xx;
    RealInterval   yy;
    double xwidth; 
    double ywidth; 

    xx = parent.vars.lookup(parent.xString);
    yy = parent.vars.lookup(parent.yString); 
    xwidth = xx.hi()-xx.lo(); 
    ywidth = yy.hi()-yy.lo();


    width = depthsize[current_depth]*boxsize_x;
    height = depthsize[current_depth]*boxsize_y;

    xratio = (i-5.0)/width;
    yratio = (j-5.0)/height;

    parent.xposition.setText(""+(xx.lo() + xratio*xwidth));
    parent.yposition.setText(""+(yy.hi() - yratio*ywidth));

  }

  public boolean mouseDown(Event e, int i, int j) {
    show_position(i,j);

    //    if (parent.zoomFlag) {
    if (true) {
      parent.buttonNarrow.disable();
      parent.buttonZoom.disable();
      parent.buttonUnZoom.disable();

      parent.gui.pushStack();
        parent.vars = parent.gui.getVars();
        parent.exp  = parent.gui.getConstraints();

      Graphics g = getGraphics();
      g.setXORMode(this.getBackground());
      zoom_x = i;
      zoom_y = j;
      last_x = i;
      last_y = j;
      parent.buttonZoom.setLabel("Zoom");
    }
    return true;
  }

  void  drawZoomRect(Graphics g,int zoom_x,int zoom_y,int last_x,int last_y) {
    if (zoom_x > last_x) { int tmp = last_x; last_x = zoom_x; zoom_x = tmp; }
    if (zoom_y > last_y) { int tmp = last_y; last_y = zoom_y; zoom_y = tmp; }
    g.setXORMode(this.getBackground());
    g.drawRect(zoom_x,zoom_y,last_x-zoom_x,last_y-zoom_y);
  }


  public boolean mouseDrag(Event e, int i, int j) {
    show_position(i,j);
    //    if (parent.zoomFlag) {
    if (true) {
      Graphics g = this.getGraphics();
      //      g.setColor(Color.black);
      drawZoomRect(g,zoom_x,zoom_y,last_x,last_y);
      //      System.out.println("\nzoom: ("+last_x+","+last_y+")");
      last_x = i;
      last_y = j;
      drawZoomRect(g,zoom_x,zoom_y,last_x,last_y);
      //      System.out.println("zoom: ("+last_x+","+last_y+")");
      g.setPaintMode();
    }
    return true;
  }

  public boolean mouseUp(Event e, int i, int j) {
    show_position(i,j);
    //    if (parent.zoomFlag) {
    if (true) {
      Graphics g = this.getGraphics();
      drawZoomRect(g,zoom_x,zoom_y,last_x,last_y);
      last_x = i;
      last_y = j;
      g.setPaintMode();
      drawZoomRect(g,zoom_x,zoom_y,last_x,last_y);
      prune(zoom_x,zoom_y,last_x,last_y);
      parent.zoomFlag = false;

      parent.buttonNarrow.enable();
      parent.buttonZoom.enable();
      if (parent.gui.sizeStack()!=0)
        parent.buttonUnZoom.enable();
    }
    return true;
  }


public boolean mouseEnter(Event e, int i, int j) {
    parent.setCursor(Frame.CROSSHAIR_CURSOR);
    return true;
}

public boolean mouseExit(Event e, int i, int j) {
    parent.xposition.setText("");
    parent.yposition.setText("");
    parent.setCursor(Frame.DEFAULT_CURSOR);
    return true;
}

public boolean mouseMove(Event e, int i, int j) {
    show_position(i,j);
    return true;
}

public void prune(int zoom_x, int zoom_y, int last_x, int last_y) {

  RealInterval xval,yval;
  double xlo,xhi,ylo,yhi,height,width;
  double newxlo,newxhi,newylo,newyhi;
  int ilo,ihi,jlo,jhi;

  if (zoom_x > last_x) {
    int tmp = last_x; last_x = zoom_x; zoom_x = tmp; }
  if (zoom_y > last_y) {
    int tmp = last_y; last_y = zoom_y; zoom_y = tmp; }


  //  System.out.println("zoom_x= "+ zoom_x + "/nlast_x = "+last_x + "width= "+plot_size.width);
  //  System.out.println("zoom_y= "+ zoom_y + "/nlast_y = "+last_y + "height= "+plot_size.height);


  xval = parent.vars.lookup(parent.xString);
  yval = parent.vars.lookup(parent.yString);
  xlo = xval.lo();
  xhi = xval.hi();
  height = xhi-xlo;

  //  System.out.println("xval = "+ xval.lo()+"-"+xval.hi() + " yval = "+yval.lo()+"-"+yval.hi());

  newxlo = xlo + 1.0*(zoom_x-5)/plot_size.width*(xhi-xlo);
  newxhi = xlo + 1.0*(last_x-5)/plot_size.width*(xhi-xlo);

  xval = new RealInterval(newxlo,newxhi);
  parent.vars.store(parent.xString,xval);


  ylo = yval.lo();
  yhi = yval.hi();
  width=yhi-ylo;

  newylo = ylo + 1.0*((plot_size.height-last_y)-5)/plot_size.height*(yhi-ylo);
  newyhi = ylo + 1.0*((plot_size.height-zoom_y)-5)/plot_size.height*(yhi-ylo);

  yval = new RealInterval(newylo,newyhi);
  parent.vars.store(parent.yString,yval);

  //  System.out.println("xval = "+ xval.lo()+"-"+xval.hi() + " yval = "+yval.lo()+"-"+yval.hi());

  parent.resetAllPlots();

  //      parent.xrange= (parent.vars.lookup(parent.xString)).toString();
  //      parent.yrange= (parent.vars.lookup(parent.yString)).toString();
  //      parent.xrangelabel.setText(parent.xrange);
  //      parent.yrangelabel.setText(parent.yrange);
  //      parent.gui.notifyNarrows(0);
  //      parent.depth = 3;
  //      parent.newPlot=true;
  //      restart();
  //      replot();
}


public void update(Graphics g) {
  paint(g);
}


synchronized void setCanvasSize() {
  int i;
  Graphics g = this.getGraphics();

      two_to_depth = 1;
      for (i = 0; i<parent.depth;i++) two_to_depth *= 2;

      canvas_size= this.size();

      //      if ((current_depth == 1)|| !parent.newPlot) {
        g.setColor(Color.white);
        g.fillRect(0,0,canvas_size.width,canvas_size.height);
	//      }
      canvas_size.width -=10;
      canvas_size.height -=10;

      if (canvas_size.width<two_to_depth) plot_size.width = two_to_depth;
      else   plot_size.width = two_to_depth*(canvas_size.width/two_to_depth);
      if (canvas_size.height<two_to_depth) plot_size.height = two_to_depth;
      else   plot_size.height = two_to_depth*(canvas_size.height/two_to_depth);
    

      //      if ((current_depth==1)|| !parent.newPlot) {
        g.setColor(Color.black);
        g.drawRect(4,4,plot_size.width+2,plot_size.height+2);
	//      }


        g.setColor(Color.white);
        if (parent.newPlot)  g.setColor(Color.red);
        g.fillRect(5,5,plot_size.width,plot_size.height);


        boxsize_x = plot_size.width/depthsize[current_depth]; 
        boxsize_y = plot_size.height/depthsize[current_depth];
        if (boxsize_x==0) boxsize_x=1;
        if (boxsize_y==0) boxsize_y=1;


}

public void paint(Graphics g) {
  Thread t = new Thread(this);
  t.start();
}

public void run() {
   if (parent.newPlot) {
     parent.newPlot = false;
     replot();
   } else {
     redraw();
   }

}





public void redraw() {
    setCanvasSize(); // set the canvas size
    if (parent.fatChoice.getSelectedIndex()==0)      
      redrawFat();
    redrawNormal();
}

public void redrawNormal() {
      int i,j;
      Graphics g = this.getGraphics();
      Enumeration tmp_enum;

      Solution sol;
    

        tmp_enum = solution_set.elements();

        g.setColor(Color.blue);
        while (tmp_enum.hasMoreElements()) {
          sol = (Solution) tmp_enum.nextElement();
          i= sol.i; j = sol.j;
          g.fillRect(5+i*boxsize_x,5+(depthsize[current_depth]-1-j)*boxsize_y,boxsize_x,boxsize_y);
        }

}


public void redrawFat() {
      int i,j;
      Graphics g = this.getGraphics();
      Enumeration tmp_enum;

      Solution sol;
    

      // first draw a fat version in red
        tmp_enum = solution_set.elements();

        g.setColor(Color.red);
        while (tmp_enum.hasMoreElements()) {
          sol = (Solution) tmp_enum.nextElement();
          i= sol.i; j = sol.j;
          g.fillRect(i*boxsize_x,(depthsize[current_depth]-1-j)*boxsize_y,boxsize_x+10,boxsize_y+10);
        }

}


public void replot() {
      int i,j;
      Graphics g = this.getGraphics();
      Solution sol;
    
      //      System.out.println("Entering replot: current_depth= "+current_depth+" parent.depth = "+parent.depth);


      if (current_depth >= parent.depth){
          current_depth = parent.depth;
          solution_set = sol_set[parent.depth];
          redraw();
          parent.buttonNarrow.enable();
          parent.buttonZoom.enable();
          if (parent.gui.sizeStack()!=0)
            parent.buttonUnZoom.enable();
          return;
      }

      setCanvasSize(); // set the canvas size

       while (current_depth < parent.depth) {
        parent.newVars = new RealIntervalTable();
        current_depth++;
	//        System.out.println(" in replot loop: current_depth= "+current_depth+" parent.depth = "+parent.depth);

        if (sol_set[current_depth]!=null) {
            solution_set = sol_set[current_depth];
            continue;
	}

        parent.messages.setText("Plotting level "+current_depth);

        boxsize_x = plot_size.width/depthsize[current_depth]; 
        boxsize_y = plot_size.height/depthsize[current_depth];

        if (boxsize_x==0) boxsize_x=1;
        if (boxsize_y==0) boxsize_y=1;

	//        System.out.println("In loop: "+boxsize_x+","+boxsize_y);

        if (current_depth==1) {
          g.setColor(Color.red);
          g.fillRect(5,5,plot_size.width,plot_size.height);
        }
        g.setColor(Color.white);
    
	//        if ((enum == null) || !enum.hasMoreElements()) {
          enum = solution_set.elements();
          solution_set = new Vector();
	  //	}
    
        while (enum.hasMoreElements()) {
          int a,b;
          sol = (Solution) enum.nextElement();
	  //          System.out.println("Enumerating ("+sol.i+","+sol.j+","+current_depth+","+depthsize[current_depth]+")");
          g.setColor(Color.black);
          g.fillRect(5+2*sol.i*boxsize_x,5+(depthsize[current_depth]-1-(2*sol.j+1))*boxsize_y,2*boxsize_x,2*boxsize_y);

          for (a=0;a<2;a++)
            for (b=0;b<2;b++) {
              i = 2*sol.i+a; 
              j = 2*sol.j+b; 

              if (parent.solvable(i,j,depthsize[current_depth])) {
                 solution_set.addElement(new Solution(i,j));
                 g.setColor(Color.blue);
                 g.fillRect(5+i*boxsize_x,5+(depthsize[current_depth]-1-j)*boxsize_y,boxsize_x,boxsize_y);
              } else {
                g.setColor(Color.red);
                g.fillRect(5+i*boxsize_x,5+(depthsize[current_depth]-1-j)*boxsize_y,boxsize_x,boxsize_y);
	      }
	    }
	}
	//       System.out.println("\n\nfinished replot: current_depth = "+current_depth +
	//                          "\n vars = \n"+parent.vars+"\n newVars = \n"+parent.newVars);
       sol_set[current_depth] = solution_set;
       redraw();
      }

       //   System.out.println("\n\nfinished replot: current_depth = "+current_depth +
       //                          "\n vars = \n"+parent.vars+"\n newVars = \n"+parent.newVars);

   redraw();
   parent.messages.setText("Select greater 'depth' for more detail.");

   if (solution_set.size()==0)
     parent.messages.setText(
    "FAILURE: the constraints have no solution in this region. Unzoom or Close the window");

   parent.buttonNarrow.enable();
   parent.buttonZoom.enable();
   if (parent.gui.sizeStack()!=0)
     parent.buttonUnZoom.enable();

  } // end of method replot

} //end of class PlotCanvas
    
    
    
    
    
    
    
    
